This is the mail archive of the gcc-patches@gcc.gnu.org mailing list for the GCC project.


Index Nav: [Date Index] [Subject Index] [Author Index] [Thread Index]
Message Nav: [Date Prev] [Date Next] [Thread Prev] [Thread Next]
Other format: [Raw text]

[PATCH GCC][4/6]Support regional coalesce and live range computation


Hi,
Following Jeff's suggestion, I am now using existing tree-ssa-live.c and
tree-ssa-coalesce.c to compute register pressure, rather than inventing
another live range solver.

The major change is to record region's basic blocks in var_map and use that
information in computation, rather than FOR_EACH_BB_FN.  For now only loop
and function type regions are supported.  The default one is function type
region which is used in out-of-ssa.  Loop type region will be used in next
patch to compute information for a loop.

Bootstrap and test on x86_64 and AArch64 ongoing.  Any comments?

Thanks,
bin
2018-04-27  Bin Cheng  <bin.cheng@arm.com>

	* tree-outof-ssa.c (remove_ssa_form): Update use.
	* tree-ssa-coalesce.c (build_ssa_conflict_graph): Support regional
	coalesce.
	(coalesce_with_default): Update comment.
	(create_outofssa_var_map): Support regional coalesce.  Rename to...
	(create_var_map): ...this.
	(coalesce_partitions): Support regional coalesce.
	(gimple_can_coalesce_p, compute_optimized_partition_bases): Ditto.
	(coalesce_ssa_name): Ditto.
	* tree-ssa-coalesce.h (coalesce_ssa_name, gimple_can_coalesce_p):
	Add parameter in declarations.
	* tree-ssa-live.c (init_var_map, delete_var_map): Support regional
	coalesce.
	(new_tree_live_info, loe_visit_block, set_var_live_on_entry): Ditto.
	(calculate_live_on_exit, verify_live_on_entry): Ditto.
	* tree-ssa-live.h (enum region_type): New.
	(struct _var_map): New fields.
	(init_var_map): Add parameter in declaration.
	(function_region_p, region_contains_p): New.
	* tree-ssa-uncprop.c (uncprop_into_successor_phis): Update uses.
From 6b7b80eb40c0bd08c25c14b3f7c33937941bdfaa Mon Sep 17 00:00:00 2001
From: Bin Cheng <binche01@e108451-lin.cambridge.arm.com>
Date: Fri, 4 May 2018 09:39:17 +0100
Subject: [PATCH 4/6] liverange-support-region-20180427

---
 gcc/tree-outof-ssa.c    |  2 +-
 gcc/tree-ssa-coalesce.c | 77 ++++++++++++++++++++++++++++++-----------------
 gcc/tree-ssa-coalesce.h |  4 +--
 gcc/tree-ssa-live.c     | 80 +++++++++++++++++++++++++++++++++++--------------
 gcc/tree-ssa-live.h     | 51 ++++++++++++++++++++++++++++++-
 gcc/tree-ssa-uncprop.c  |  5 ++--
 6 files changed, 163 insertions(+), 56 deletions(-)

diff --git a/gcc/tree-outof-ssa.c b/gcc/tree-outof-ssa.c
index 59bdcd6..81edbc5 100644
--- a/gcc/tree-outof-ssa.c
+++ b/gcc/tree-outof-ssa.c
@@ -945,7 +945,7 @@ remove_ssa_form (bool perform_ter, struct ssaexpand *sa)
   bitmap values = NULL;
   var_map map;
 
-  map = coalesce_ssa_name ();
+  map = coalesce_ssa_name (NULL, flag_tree_coalesce_vars);
 
   /* Return to viewing the variable list as just all reference variables after
      coalescing has been performed.  */
diff --git a/gcc/tree-ssa-coalesce.h b/gcc/tree-ssa-coalesce.h
index 89d8474..66acb18 100644
--- a/gcc/tree-ssa-coalesce.h
+++ b/gcc/tree-ssa-coalesce.h
@@ -20,8 +20,8 @@ along with GCC; see the file COPYING3.  If not see
 #ifndef GCC_TREE_SSA_COALESCE_H
 #define GCC_TREE_SSA_COALESCE_H
 
-extern var_map coalesce_ssa_name (void);
-extern bool gimple_can_coalesce_p (tree, tree);
+extern var_map coalesce_ssa_name (struct loop*, bool);
+extern bool gimple_can_coalesce_p (tree, tree, bool);
 extern bitmap get_parm_default_def_partitions (var_map);
 extern bitmap get_undefined_value_partitions (var_map);
 
diff --git a/gcc/tree-ssa-live.h b/gcc/tree-ssa-live.h
index 448aaf9..fa6f68d 100644
--- a/gcc/tree-ssa-live.h
+++ b/gcc/tree-ssa-live.h
@@ -42,6 +42,16 @@ along with GCC; see the file COPYING3.  If not see
 
    Note that members of a partition MUST all have the same base variable.  */
 
+/* The type of region within which live range is computed.  For now we only
+   support loop and function type regions.  */
+enum region_type
+{
+  RTYPE_BB,
+  RTYPE_LOOP,
+  RTYPE_SESE,
+  RTYPE_FUNC
+};
+
 typedef struct _var_map
 {
   /* The partition manager of all variables.  */
@@ -62,13 +72,27 @@ typedef struct _var_map
 
   /* Map of partitions numbers to base variable table indexes.  */
   int *partition_to_base_index;
+
+  /* Bitmap of basic block.  It describes the region within which the analysis
+     is done.  */
+  bitmap bmp_bbs;
+
+  /* Vector of basic block in the region.  */
+  vec<basic_block> *vec_bbs;
+
+  /* Type of region.  */
+  enum region_type region_type;
+
+  /* Attemp to reduce copying by coalescing versions of user defined variables
+     if TRUE.  */
+  bool coalesce_vars_p;
 } *var_map;
 
 
 /* Value used to represent no partition number.  */
 #define NO_PARTITION		-1
 
-extern var_map init_var_map (int);
+extern var_map init_var_map (struct loop*, int, bool);
 extern void delete_var_map (var_map);
 extern int var_union (var_map, tree, tree);
 extern void partition_view_normal (var_map);
@@ -82,6 +106,31 @@ extern void debug (_var_map &ref);
 extern void debug (_var_map *ptr);
 
 
+/* Return TRUE if region of the MAP is the whole function.  */
+
+inline bool
+function_region_p (var_map map)
+{
+  return map->region_type == RTYPE_FUNC;
+}
+
+
+/* Return TRUE if region of the MAP contains basic block BB.  */
+
+inline bool
+region_contains_p (var_map map, basic_block bb)
+{
+  if (bb == ENTRY_BLOCK_PTR_FOR_FN (cfun)
+      || bb == EXIT_BLOCK_PTR_FOR_FN (cfun))
+    return false;
+
+  if (function_region_p (map))
+    return true;
+
+  return bitmap_bit_p (map->bmp_bbs, bb->index);
+}
+
+
 /* Return number of partitions in MAP.  */
 
 static inline unsigned
diff --git a/gcc/tree-ssa-uncprop.c b/gcc/tree-ssa-uncprop.c
index 7d863a7..89863de 100644
--- a/gcc/tree-ssa-uncprop.c
+++ b/gcc/tree-ssa-uncprop.c
@@ -374,7 +374,7 @@ uncprop_into_successor_phis (basic_block bb)
 	     coalesced with the result, then there's no point in
 	     un-propagating the argument.  */
 	  if (!is_gimple_min_invariant (arg)
-	      && gimple_can_coalesce_p (arg, res))
+	      && gimple_can_coalesce_p (arg, res, flag_tree_coalesce_vars))
 	    continue;
 
 	  /* Lookup this argument's value in the hash table.  */
@@ -390,7 +390,8 @@ uncprop_into_successor_phis (basic_block bb)
 		{
 		  tree equiv = (*equivalences)[j];
 
-		  if (gimple_can_coalesce_p (equiv, res))
+		  if (gimple_can_coalesce_p (equiv, res,
+					     flag_tree_coalesce_vars))
 		    {
 		      SET_PHI_ARG_DEF (phi, e->dest_idx, equiv);
 		      break;
diff --git a/gcc/tree-ssa-coalesce.c b/gcc/tree-ssa-coalesce.c
index 5cc0aca..7269eb1 100644
--- a/gcc/tree-ssa-coalesce.c
+++ b/gcc/tree-ssa-coalesce.c
@@ -869,7 +869,7 @@ build_ssa_conflict_graph (tree_live_info_p liveinfo)
      coalesce variables from different base variables, including
      different parameters, so we have to make sure default defs live
      at the entry block conflict with each other.  */
-  if (flag_tree_coalesce_vars)
+  if (liveinfo->map->coalesce_vars_p)
     entry = single_succ (ENTRY_BLOCK_PTR_FOR_FN (cfun));
   else
     entry = NULL;
@@ -879,7 +879,7 @@ build_ssa_conflict_graph (tree_live_info_p liveinfo)
 
   live = new_live_track (map);
 
-  FOR_EACH_BB_FN (bb, cfun)
+  for (unsigned i = 0; liveinfo->map->vec_bbs->iterate (i, &bb); ++i)
     {
       /* Start with live on exit temporaries.  */
       live_track_init (live, live_on_exit (liveinfo, bb));
@@ -944,6 +944,8 @@ build_ssa_conflict_graph (tree_live_info_p liveinfo)
 	{
 	  gphi *phi = gsi.phi ();
 	  tree result = PHI_RESULT (phi);
+	  if (virtual_operand_p (result))
+	    continue;
 	  if (live_track_live_p (live, result))
 	    live_track_process_def (live, result, graph);
 	}
@@ -1071,14 +1073,18 @@ coalesce_with_default (tree var, coalesce_list *cl, bitmap used_in_copy)
   add_cost_one_coalesce (cl, SSA_NAME_VERSION (ssa), SSA_NAME_VERSION (var));
   bitmap_set_bit (used_in_copy, SSA_NAME_VERSION (var));
   /* Default defs will have their used_in_copy bits set at the end of
-     create_outofssa_var_map.  */
+     create_var_map.  */
 }
 
-/* This function creates a var_map for the current function as well as creating
-   a coalesce list for use later in the out of ssa process.  */
+/* This function creates a var_map for a region indicated by BBS in the current
+   function as well as creating a coalesce list for use later in the out of ssa
+   process.  Region is a loop if LOOP is not NULL, otherwise the function.
+   COALESCE_VARS_P is true if we coalesce version of different user-defined
+   variables.  */
 
 static var_map
-create_outofssa_var_map (coalesce_list *cl, bitmap used_in_copy)
+create_var_map (struct loop *loop, coalesce_list *cl, bitmap used_in_copy,
+		bool coalesce_vars_p)
 {
   gimple_stmt_iterator gsi;
   basic_block bb;
@@ -1091,11 +1097,11 @@ create_outofssa_var_map (coalesce_list *cl, bitmap used_in_copy)
 
   for_all_parms (create_default_def, NULL);
 
-  map = init_var_map (num_ssa_names);
+  map = init_var_map (loop, num_ssa_names, coalesce_vars_p);
 
   for_all_parms (register_default_def, NULL);
 
-  FOR_EACH_BB_FN (bb, cfun)
+  for (unsigned j = 0; map->vec_bbs->iterate (j, &bb); ++j)
     {
       tree arg;
 
@@ -1110,6 +1116,8 @@ create_outofssa_var_map (coalesce_list *cl, bitmap used_in_copy)
 	  bool saw_copy = false;
 
 	  res = gimple_phi_result (phi);
+	  if (virtual_operand_p (res))
+	    continue;
 	  ver = SSA_NAME_VERSION (res);
 
 	  /* Register ssa_names and coalesces between the args and the result
@@ -1121,7 +1129,7 @@ create_outofssa_var_map (coalesce_list *cl, bitmap used_in_copy)
 	      if (TREE_CODE (arg) != SSA_NAME)
 		continue;
 
-	      if (gimple_can_coalesce_p (arg, res)
+	      if (gimple_can_coalesce_p (arg, res, coalesce_vars_p)
 		  || (e->flags & EDGE_ABNORMAL))
 		{
 		  saw_copy = true;
@@ -1155,7 +1163,7 @@ create_outofssa_var_map (coalesce_list *cl, bitmap used_in_copy)
 		tree lhs = gimple_assign_lhs (stmt);
 		tree rhs1 = gimple_assign_rhs1 (stmt);
 		if (gimple_assign_ssa_name_copy_p (stmt)
-		    && gimple_can_coalesce_p (lhs, rhs1))
+		    && gimple_can_coalesce_p (lhs, rhs1, coalesce_vars_p))
 		  {
 		    v1 = SSA_NAME_VERSION (lhs);
 		    v2 = SSA_NAME_VERSION (rhs1);
@@ -1179,7 +1187,7 @@ create_outofssa_var_map (coalesce_list *cl, bitmap used_in_copy)
 		tree lhs = ssa_default_def (cfun, res);
 		gcc_assert (lhs);
 		if (TREE_CODE (rhs1) == SSA_NAME
-		    && gimple_can_coalesce_p (lhs, rhs1))
+		    && gimple_can_coalesce_p (lhs, rhs1, coalesce_vars_p))
 		  {
 		    v1 = SSA_NAME_VERSION (lhs);
 		    v2 = SSA_NAME_VERSION (rhs1);
@@ -1231,7 +1239,8 @@ create_outofssa_var_map (coalesce_list *cl, bitmap used_in_copy)
 		    v1 = SSA_NAME_VERSION (outputs[match]);
 		    v2 = SSA_NAME_VERSION (input);
 
-		    if (gimple_can_coalesce_p (outputs[match], input))
+		    if (gimple_can_coalesce_p (outputs[match], input,
+					       coalesce_vars_p))
 		      {
 			cost = coalesce_cost (REG_BR_PROB_BASE,
 					      optimize_bb_for_size_p (bb));
@@ -1249,6 +1258,9 @@ create_outofssa_var_map (coalesce_list *cl, bitmap used_in_copy)
 	}
     }
 
+  if (!function_region_p (map))
+    return map;
+
   /* Now process result decls and live on entry variables for entry into
      the coalesce list.  */
   first = NULL_TREE;
@@ -1267,7 +1279,8 @@ create_outofssa_var_map (coalesce_list *cl, bitmap used_in_copy)
 		first = var;
 	      else
 		{
-		  gcc_assert (gimple_can_coalesce_p (var, first));
+		  gcc_assert (gimple_can_coalesce_p (var, first,
+						     coalesce_vars_p));
 		  v1 = SSA_NAME_VERSION (first);
 		  v2 = SSA_NAME_VERSION (var);
 		  cost = coalesce_cost_bb (EXIT_BLOCK_PTR_FOR_FN (cfun));
@@ -1384,13 +1397,15 @@ coalesce_partitions (var_map map, ssa_conflicts *graph, coalesce_list *cl,
 		 gsi_next (&gsi))
 	      {
 		gphi *phi = gsi.phi ();
+		tree res = PHI_RESULT (phi);
+		if (virtual_operand_p (res))
+		  continue;
 		tree arg = PHI_ARG_DEF (phi, e->dest_idx);
 		if (SSA_NAME_IS_DEFAULT_DEF (arg)
 		    && (!SSA_NAME_VAR (arg)
 			|| TREE_CODE (SSA_NAME_VAR (arg)) != PARM_DECL))
 		  continue;
 
-		tree res = PHI_RESULT (phi);
 		int v1 = SSA_NAME_VERSION (res);
 		int v2 = SSA_NAME_VERSION (arg);
 
@@ -1411,7 +1426,7 @@ coalesce_partitions (var_map map, ssa_conflicts *graph, coalesce_list *cl,
       var2 = ssa_name (y);
 
       /* Assert the coalesces have the same base variable.  */
-      gcc_assert (gimple_can_coalesce_p (var1, var2));
+      gcc_assert (gimple_can_coalesce_p (var1, var2, map->coalesce_vars_p));
 
       if (debug)
 	fprintf (debug, "Coalesce list: ");
@@ -1493,13 +1508,15 @@ dump_part_var_map (FILE *f, partition part, var_map map)
 }
 
 /* Given SSA_NAMEs NAME1 and NAME2, return true if they are candidates for
-   coalescing together, false otherwise.
+   coalescing together, false otherwise.  If COALESCE_VARS_P is TRUE, we
+   try to coalesce versions of different user-defined variables.  Normally
+   -ftree-coalesce-vars is passed in.
 
    This must stay consistent with compute_samebase_partition_bases and 
    compute_optimized_partition_bases.  */
 
 bool
-gimple_can_coalesce_p (tree name1, tree name2)
+gimple_can_coalesce_p (tree name1, tree name2, bool coalesce_vars_p)
 {
   /* First check the SSA_NAME's associated DECL.  Without
      optimization, we only want to coalesce if they have the same DECL
@@ -1508,7 +1525,7 @@ gimple_can_coalesce_p (tree name1, tree name2)
   tree var2 = SSA_NAME_VAR (name2);
   var1 = (var1 && (!VAR_P (var1) || !DECL_IGNORED_P (var1))) ? var1 : NULL_TREE;
   var2 = (var2 && (!VAR_P (var2) || !DECL_IGNORED_P (var2))) ? var2 : NULL_TREE;
-  if (var1 != var2 && !flag_tree_coalesce_vars)
+  if (var1 != var2 && !coalesce_vars_p)
     return false;
 
   /* Now check the types.  If the types are the same, then we should
@@ -1565,7 +1582,7 @@ gimple_can_coalesce_p (tree name1, tree name2)
 
      In the non-optimized case, we must first test TYPE_CANONICAL because
      we use it to compute the partition_to_base_index of the map.  */
-  if (flag_tree_coalesce_vars)
+  if (coalesce_vars_p)
     {
       if (types_compatible_p (t1, t2))
 	goto check_modes;
@@ -1629,8 +1646,9 @@ compute_optimized_partition_bases (var_map map, bitmap used_in_copies,
   /* And also with abnormal edges.  */
   basic_block bb;
   edge e;
+  unsigned i;
   edge_iterator ei;
-  FOR_EACH_BB_FN (bb, cfun)
+  for (i = 0; map->vec_bbs->iterate (i, &bb); ++i)
     {
       FOR_EACH_EDGE (e, ei, bb->preds)
 	if (e->flags & EDGE_ABNORMAL)
@@ -1640,14 +1658,15 @@ compute_optimized_partition_bases (var_map map, bitmap used_in_copies,
 		 gsi_next (&gsi))
 	      {
 		gphi *phi = gsi.phi ();
+		tree res = PHI_RESULT (phi);
+		if (virtual_operand_p (res))
+		  continue;
 		tree arg = PHI_ARG_DEF (phi, e->dest_idx);
 		if (SSA_NAME_IS_DEFAULT_DEF (arg)
 		    && (!SSA_NAME_VAR (arg)
 			|| TREE_CODE (SSA_NAME_VAR (arg)) != PARM_DECL))
 		  continue;
 
-		tree res = PHI_RESULT (phi);
-
 		int p1 = partition_find (tentative, var_to_partition (map, res));
 		int p2 = partition_find (tentative, var_to_partition (map, arg));
 
@@ -1675,7 +1694,6 @@ compute_optimized_partition_bases (var_map map, bitmap used_in_copies,
      between all SSA versions that ended up in the same potential
      coalesce partition.  */
   bitmap_iterator bi;
-  unsigned i;
   EXECUTE_IF_SET_IN_BITMAP (used_in_copies, 0, i, bi)
     {
       int pidx = var_to_partition (map, ssa_name (i));
@@ -1785,10 +1803,13 @@ compute_samebase_partition_bases (var_map map)
 }
 
 /* Reduce the number of copies by coalescing variables in the function.  Return
-   a partition map with the resulting coalesces.  */
+   a partition map with the resulting coalesces.  The coalesce is done on a
+   region basis; and the region is a loop if LOOP is not NULL, otherwise is the
+   function.  COALESCE_VARS_P is true if we coalesce version of different
+   user-defined variables.  */
 
 extern var_map
-coalesce_ssa_name (void)
+coalesce_ssa_name (struct loop *loop, bool coalesce_vars_p)
 {
   tree_live_info_p liveinfo;
   ssa_conflicts *graph;
@@ -1799,12 +1820,12 @@ coalesce_ssa_name (void)
   tree a;
 
   cl = create_coalesce_list ();
-  map = create_outofssa_var_map (cl, used_in_copies);
+  map = create_var_map (loop, cl, used_in_copies, coalesce_vars_p);
 
   /* If this optimization is disabled, we need to coalesce all the
      names originating from the same SSA_NAME_VAR so debug info
      remains undisturbed.  */
-  if (!flag_tree_coalesce_vars)
+  if (!map->coalesce_vars_p)
     {
       hash_table<ssa_name_var_hash> ssa_name_hash (10);
 
@@ -1845,7 +1866,7 @@ coalesce_ssa_name (void)
 
   partition_view_bitmap (map, used_in_copies);
 
-  if (flag_tree_coalesce_vars)
+  if (map->coalesce_vars_p)
     compute_optimized_partition_bases (map, used_in_copies, cl);
   else
     compute_samebase_partition_bases (map);
diff --git a/gcc/tree-ssa-live.c b/gcc/tree-ssa-live.c
index 62316ba..ccb0d99 100644
--- a/gcc/tree-ssa-live.c
+++ b/gcc/tree-ssa-live.c
@@ -71,10 +71,13 @@ var_map_base_fini (var_map map)
       map->num_basevars = 0;
     }
 }
-/* Create a variable partition map of SIZE, initialize and return it.  */
+/* Create a variable partition map of SIZE for region, initialize and return
+   it.  Region is a loop if LOOP is non-NULL, otherwise is current function.
+   COALESCE_VARS_P is true if we coalesce versions of different user-defined
+   variables.  */
 
 var_map
-init_var_map (int size)
+init_var_map (struct loop *loop, int size, bool coalesce_vars_p)
 {
   var_map map;
 
@@ -87,6 +90,30 @@ init_var_map (int size)
   map->partition_size = size;
   map->num_basevars = 0;
   map->partition_to_base_index = NULL;
+  map->bmp_bbs = BITMAP_ALLOC (NULL);
+  map->vec_bbs = new vec<basic_block> ();
+  if (loop)
+    {
+      map->region_type = RTYPE_LOOP;
+      basic_block *bbs = get_loop_body_in_dom_order (loop);
+      for (unsigned i = 0; i < loop->num_nodes; ++i)
+	{
+	  bitmap_set_bit (map->bmp_bbs, bbs[i]->index);
+	  map->vec_bbs->safe_push (bbs[i]);
+	}
+      free (bbs);
+    }
+  else
+    {
+      map->region_type = RTYPE_FUNC;
+      basic_block bb;
+      FOR_EACH_BB_FN (bb, cfun)
+	{
+	  bitmap_set_bit (map->bmp_bbs, bb->index);
+	  map->vec_bbs->safe_push (bb);
+	}
+    }
+  map->coalesce_vars_p = coalesce_vars_p;
   return map;
 }
 
@@ -100,6 +127,9 @@ delete_var_map (var_map map)
   partition_delete (map->var_partition);
   free (map->partition_to_view);
   free (map->view_to_partition);
+  BITMAP_FREE (map->bmp_bbs);
+  map->vec_bbs->release ();
+  delete map->vec_bbs;
   free (map);
 }
 
@@ -901,13 +931,14 @@ new_tree_live_info (var_map map)
 
   bitmap_obstack_initialize (&live->livein_obstack);
   bitmap_obstack_initialize (&live->liveout_obstack);
-  live->livein = XNEWVEC (bitmap_head, last_basic_block_for_fn (cfun));
-  FOR_EACH_BB_FN (bb, cfun)
-    bitmap_initialize (&live->livein[bb->index], &live->livein_obstack);
 
-  live->liveout = XNEWVEC (bitmap_head, last_basic_block_for_fn (cfun));
-  FOR_EACH_BB_FN (bb, cfun)
-    bitmap_initialize (&live->liveout[bb->index], &live->liveout_obstack);
+  live->livein = XCNEWVEC (bitmap_head, last_basic_block_for_fn (cfun));
+  live->liveout = XCNEWVEC (bitmap_head, last_basic_block_for_fn (cfun));
+  for (unsigned i = 0; map->vec_bbs->iterate (i, &bb); ++i)
+    {
+      bitmap_initialize (&live->livein[bb->index], &live->livein_obstack);
+      bitmap_initialize (&live->liveout[bb->index], &live->liveout_obstack);
+    }
 
   live->work_stack = XNEWVEC (int, last_basic_block_for_fn (cfun));
   live->stack_top = live->work_stack;
@@ -960,7 +991,7 @@ loe_visit_block (tree_live_info_p live, basic_block bb, sbitmap visited)
   FOR_EACH_EDGE (e, ei, bb->preds)
     {
       pred_bb = e->src;
-      if (pred_bb == ENTRY_BLOCK_PTR_FOR_FN (cfun))
+      if (!region_contains_p (live->map, pred_bb))
 	continue;
       /* Variables live-on-entry from BB that aren't defined in the
 	 predecessor block.  This should be the live on entry vars to pred.
@@ -993,9 +1024,10 @@ live_worklist (tree_live_info_p live)
 
   bitmap_clear (visited);
 
   /* Visit all the blocks in reverse order and propagate live on entry values
      into the predecessors blocks.  */
-  FOR_EACH_BB_REVERSE_FN (bb, cfun)
+  for (unsigned i = live->map->vec_bbs->length () - 1;
+       live->map->vec_bbs->iterate (i, &bb); --i)
     loe_visit_block (live, bb, visited);
 
   /* Process any blocks which require further iteration.  */
@@ -1030,7 +1062,7 @@ set_var_live_on_entry (tree ssa_name, tree_live_info_p live)
     {
       def_bb = gimple_bb (stmt);
       /* Mark defs in liveout bitmap temporarily.  */
-      if (def_bb)
+      if (def_bb && region_contains_p (live->map, def_bb))
 	bitmap_set_bit (&live->liveout[def_bb->index], p);
     }
   else
@@ -1054,11 +1086,8 @@ set_var_live_on_entry (tree ssa_name, tree_live_info_p live)
 	     defined in that block, or whether its live on entry.  */
 	  int index = PHI_ARG_INDEX_FROM_USE (use);
 	  edge e = gimple_phi_arg_edge (as_a <gphi *> (use_stmt), index);
-	  if (e->src != ENTRY_BLOCK_PTR_FOR_FN (cfun))
-	    {
-	      if (e->src != def_bb)
-		add_block = e->src;
-	    }
+	  if (e->src != def_bb && region_contains_p (live->map, e->src))
+	    add_block = e->src;
 	}
       else if (is_gimple_debug (use_stmt))
 	continue;
@@ -1066,7 +1095,7 @@ set_var_live_on_entry (tree ssa_name, tree_live_info_p live)
         {
 	  /* If its not defined in this block, its live on entry.  */
 	  basic_block use_bb = gimple_bb (use_stmt);
-	  if (use_bb != def_bb)
+	  if (use_bb != def_bb && region_contains_p (live->map, use_bb))
 	    add_block = use_bb;
 	}
 
@@ -1095,7 +1124,7 @@ calculate_live_on_exit (tree_live_info_p liveinfo)
   edge_iterator ei;
 
   /* live on entry calculations used liveout vectors for defs, clear them.  */
-  FOR_EACH_BB_FN (bb, cfun)
+  for (unsigned i = 0; liveinfo->map->vec_bbs->iterate (i, &bb); ++i)
     bitmap_clear (&liveinfo->liveout[bb->index]);
 
   /* Set all the live-on-exit bits for uses in PHIs.  */
@@ -1108,6 +1137,8 @@ calculate_live_on_exit (tree_live_info_p liveinfo)
       for (gsi = gsi_start_phis (bb); !gsi_end_p (gsi); gsi_next (&gsi))
 	{
 	  gphi *phi = gsi.phi ();
+	  if (virtual_operand_p (gimple_phi_result (phi)))
+	    continue;
 	  for (i = 0; i < gimple_phi_num_args (phi); i++)
 	    {
 	      tree t = PHI_ARG_DEF (phi, i);
@@ -1120,14 +1151,17 @@ calculate_live_on_exit (tree_live_info_p liveinfo)
 	      if (p == NO_PARTITION)
 		continue;
 	      e = gimple_phi_arg_edge (phi, i);
-	      if (e->src != ENTRY_BLOCK_PTR_FOR_FN (cfun))
+	      if (region_contains_p (liveinfo->map, e->src))
 		bitmap_set_bit (&liveinfo->liveout[e->src->index], p);
 	    }
 	}
 
+      if (!region_contains_p (liveinfo->map, bb))
+	continue;
+
       /* Add each successors live on entry to this bock live on exit.  */
       FOR_EACH_EDGE (e, ei, bb->succs)
-	if (e->dest != EXIT_BLOCK_PTR_FOR_FN (cfun))
+	if (region_contains_p (liveinfo->map, e->dest))
 	  bitmap_ior_into (&liveinfo->liveout[bb->index],
 			   live_on_entry (liveinfo, e->dest));
     }
@@ -1314,7 +1348,7 @@ verify_live_on_entry (tree_live_info_p live)
   FOR_EACH_EDGE (e, ei, bb->succs)
     {
       int entry_block = e->dest->index;
-      if (e->dest == EXIT_BLOCK_PTR_FOR_FN (cfun))
+      if (!region_contains_p (live->map, e->dest))
         continue;
       for (i = 0; i < (unsigned)num_var_partitions (map); i++)
 	{
@@ -1380,6 +1414,8 @@ verify_live_on_entry (tree_live_info_p live)
 		     gsi_next (&gsi))
 		  {
 		    gphi *phi = gsi.phi ();
+		    if (virtual_operand_p (gimple_phi_result (phi)))
+		      continue;
 		    for (z = 0; z < gimple_phi_num_args (phi); z++)
 		      if (var == gimple_phi_arg_def (phi, z))
 			{
@@ -993,9 +1024,10 @@ live_worklist (tree_live_info_p live)
   bitmap_clear (visited);
 
-  /* Visit all the blocks in reverse order and propagate live on entry values
+  /* Visit region's blocks in reverse order and propagate live on entry values
      into the predecessors blocks.  */
-- 
1.9.1


Index Nav: [Date Index] [Subject Index] [Author Index] [Thread Index]
Message Nav: [Date Prev] [Date Next] [Thread Prev] [Thread Next]